<!--
- SPDX-License-Identifier: Apache-2.0
- Copyright (C) 2025 Jayesh Badwaik <j.badwaik@fz-juelich.de>
-->

# logmap - a microbenchmark with controllable arithmetic intensity

A [logistic map](https://en.wikipedia.org/wiki/Logistic_map) is a simple
mathematical model that exhibits chaotic behavior. This repository contains a
microbenchmark that is based on the logistic map that can be controlled to have
a desired arbitrary workload and desired arbitrary arithmetic intensity.

The microbenchmark works by iterating each element of an array of floating-point
numbers `x` of size `N` using the logistic map formula for a
specified number of iterations `niter`. The logistic map formula is given by:


```math
x_{i, n+1} = r x_{i,n} (1 - x_{i,n})
```

where $i$ is the index of the element in the array and $n$ is the iteration
number. The value of `r` is a constant that determines the behavior of the
logistic map. The value of `r` is selected to be within the range [3.5, 4.0] to
ensure that the logistic map exhibits chaotic behavior. The chaotic nature of
the logistic map ensures that the benchmark hardware is not susceptible to
effects of [input-dependent power
consumption](https://arxiv.org/abs/2409.18324).

The arithmetic intensity of the benchmark is directly proportional to the number
of iterations `niter` and independent of the size of the array `N` and can be
written as

```math
AI = k_i niter
```

where `k_i` is the arithmetic intensity  for a single iteration of the logistic
map for a single input. The workload of the benchmark is directly proportional
to the size of the array `N` and directly proportional to the number of
iterations `niter` and can be written as

```math
W = k_w N niter
```

where `k_w` is the floating-point operations per element of the array `x` for a
single iteration of the logistic map.



